@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,31 +18,29 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ArgV.h,v 1.12 2001/10/12 10:50:06 lav Exp $ */
+/* $Id: ArgV.h,v 1.17 2004/02/17 10:44:19 lav Exp $ */
 
 #ifndef ARGV_H
 #define ARGV_H
 
-class ArgV
-{
-   char **v;
-   int c;
-   int ind;
-   int allocated;
+#include <stdio.h>
+#include "StringSet.h"
 
-   void Init(int,const char * const *);
-   void GetRoom(int);
+class ArgV : public StringSet
+{
+   int ind;
 
 public:
-   ArgV() { Init(0,0); }
-   ArgV(const char *a0) { Init(1,&a0); }
+   ArgV() { ind=0; }
+   ArgV(const char *a0) : StringSet(&a0,1) { ind=0; }
    ArgV(const char *a0, const char *args);
-   void Empty();
-   void Append(const char *);
+   ArgV(const ArgV& a) : StringSet(a) { ind=0; }
+   ArgV(const ArgV *a) : StringSet(*a) { ind=0; }
+   ArgV(int new_c,const char * const *new_v) : StringSet(new_v,new_c) { ind=0; }
+
+   void Append(const char *s) { StringSet::Append(s); }
+   void Append(int a) { char buf[12]; sprintf(buf,"%d",a); Append(buf); }
    void Add(const char *a) { Append(a); } // alias
-   ArgV(const ArgV& a) { Init(a.c,a.v); }
-   ArgV(int new_c,const char * const *new_v) { Init(new_c,new_v); }
-   ~ArgV();
 
    char *Combine(int start_index=0) const;
    char *CombineQuoted(int start_index=0) const;
@@ -53,24 +51,21 @@ public:
 	 return getopt_long(opts,0,0);
       }
 
-   void rewind();
-   char *getnext();
+   void seek(int n);
+   void rewind() { seek(0); }
+   const char *getnext();
 
-   char *getarg(int n) const
-      {
-	 if(n>=c)
-	    return 0;
-	 return v[n];
-      }
-   char *getcurr() const { return ind<c?getarg(ind):0; }
+   const char *getarg(int n) const { return String(n); }
+   const char *getcurr() const { return ind<Count()?getarg(ind):0; }
    int getindex() const { return ind; }
-   void setarg(int n,const char *s);
-   void delarg(int n);
-   void insarg(int n,const char *s);
-   char *a0() const { return getarg(0); }
+   void setarg(int n,const char *s) { Replace(n,s); }
+   void delarg(int n) { if(ind>n)--ind; Remove(n); }
+   void insarg(int n,const char *s) { InsertBefore(n,s); }
+   const char *a0() const { return getarg(0); }
    void back();
-   int count() const { return c; }
-   char **GetV() const { return v; }
+   int count() const { return Count(); }
+   const char *const*GetV() const { return Set(); }
+   char **GetVNonConst() { return SetNonConst(); }
 };
 
 #endif//ARGV_H
