@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.h,v 1.50 2002/07/10 08:48:51 lav Exp $ */
+/* $Id: FileCopy.h,v 1.54 2003/12/01 07:46:18 lav Exp $ */
 
 /*
   classes defined here:
@@ -37,14 +37,16 @@
 #include "buffer.h"
 #include "FileAccess.h"
 #include "Speedometer.h"
+#include "Timer.h"
 
 #define FILE_END     (-1L)
 
-class FileCopyPeer : public Buffer
+class FileCopyPeer : public IOBuffer
 {
 protected:
    bool want_size;
    bool want_date;
+   bool start_transfer;
    off_t size;
    off_t e_size;
    time_t date;
@@ -65,14 +67,10 @@ protected:
    bool write_allowed;
 
 public:
-   enum direction { GET, PUT };
-
    off_t range_start; // NOTE: ranges are implemented only partially. (FIXME)
    off_t range_limit;
 
 protected:
-   enum direction mode;
-
    ~FileCopyPeer();
 
 public:
@@ -99,7 +97,7 @@ public:
 
    void SetRange(off_t s,off_t lim) { range_start=s; range_limit=lim; }
 
-   FileCopyPeer(direction m);
+   FileCopyPeer(dir_t m);
 
    bool Done();
 
@@ -130,6 +128,11 @@ public:
    bool WritePending() { return mode==PUT && Size()>0; }
 
    bool FileRemoved() { return file_removed; }
+
+   void DontStartTransferYet() { start_transfer=false; }
+   void StartTransfer() { start_transfer=true; }
+
+   const char *GetDescriptionForLog() { return 0; }
 };
 
 class FileCopy : public SMTask
@@ -297,12 +300,18 @@ public:
    void Fg() { session->SetPriority(1); }
    void Bg() { session->SetPriority(0); }
    void SetFXP(bool on) { fxp=on; }
+
+   const char *GetDescriptionForLog()
+      {
+	 return orig_url ? orig_url : session->GetFileURL(file);
+      }
 };
 
 class FileCopyPeerFDStream : public FileCopyPeer
 {
    FDStream *stream;
    off_t seek_base;
+   Timer *put_ll_timer;
 
    int Get_LL(int size);
    int Put_LL(const char *buf,int size);
@@ -318,7 +327,7 @@ protected:
    ~FileCopyPeerFDStream();
 
 public:
-   FileCopyPeerFDStream(FDStream *o,direction m);
+   FileCopyPeerFDStream(FDStream *o,dir_t m);
    int Do();
    bool IOReady();
    void Seek(off_t new_pos);
@@ -336,6 +345,11 @@ public:
 
    static FileCopyPeerFDStream *NewPut(const char *file,bool cont=false);
    static FileCopyPeerFDStream *NewGet(const char *file);
+
+   const char *GetDescriptionForLog()
+      {
+	 return stream->name;
+      }
 };
 
 class FileCopyPeerDirList : public FileCopyPeer
@@ -371,6 +385,11 @@ public:
    int Do();
    void Fg();
    void Bg();
+
+   const char *GetDescriptionForLog()
+      {
+	 return "[pipe to other job]";
+      }
 };
 
 #endif
