@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Fish.h,v 1.15 2001/11/08 09:25:48 lav Exp $ */
+/* $Id: Fish.h,v 1.19 2003/12/01 07:46:19 lav Exp $ */
 
 #ifndef FISH_H
 #define FISH_H
@@ -106,7 +106,7 @@ class Fish : public NetAccess
    int	 RespQueueSize() { return RQ_tail-RQ_head; }
    void  EmptyRespQueue() { RQ_head=RQ_tail=0; }
 
-   void GetBetterConnection(int level,int count);
+   void GetBetterConnection(int level);
    void MoveConnectionHere(Fish *o);
 
    char  *line;
@@ -144,7 +144,8 @@ public:
 
    DirList *MakeDirList(ArgV *args);
    Glob *MakeGlob(const char *pattern);
-   ListInfo *MakeListInfo();
+   ListInfo *MakeListInfo(const char *p);
+   FileSet *ParseLongList(const char *buf,int len,int *err=0);
 
    static const char *shell_encode(const char *);
    void DontEncodeFile() { encode_file=false; }
@@ -161,7 +162,7 @@ public:
 class FishDirList : public DirList
 {
    FileAccess *session;
-   Buffer *ubuf;
+   IOBuffer *ubuf;
    char *pattern;
 
 public:
@@ -174,21 +175,12 @@ public:
    void Resume();
 };
 
-class FishGlob : public GenericParseGlob
-{
-   FileSet *Parse(const char *buf,int len);
-   GenericParseGlob *MakeUpDirGlob();
-public:
-   FishGlob(FileAccess *s,const char *n_pattern)
-      : GenericParseGlob(s,n_pattern) {}
-};
-
 class FishListInfo : public GenericParseListInfo
 {
    FileSet *Parse(const char *buf,int len);
 public:
-   FishListInfo(Fish *session)
-      : GenericParseListInfo(session)
+   FishListInfo(Fish *session,const char *path)
+      : GenericParseListInfo(session,path)
       {
 	 can_get_prec_time=false;
       }
