@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.155 2002/09/05 09:37:51 lav Exp $ */
+/* $Id: Http.cc,v 1.170 2004/05/28 07:57:04 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -42,7 +42,7 @@
 
 #include "ascii_ctype.h"
 
-#ifndef HAVE_STRPTIME_DECL
+#if !HAVE_DECL_STRPTIME
 CDECL char *strptime(const char *buf, const char *format, struct tm *tm);
 #endif
 
@@ -83,6 +83,7 @@ void Http::Init()
    proto_version=0x10;
    sent_eot=false;
    last_method=0;
+   entity_content_type=0;
 
    default_cwd="/";
 
@@ -197,6 +198,8 @@ void Http::ResetRequestData()
    chunk_pos=0;
    chunked_trailer=false;
    seen_ranges_bytes=false;
+   xfree(entity_content_type);
+   entity_content_type=0;
 }
 
 void Http::Close()
@@ -298,6 +301,16 @@ void Http::SendMethod(const char *method,const char *efile)
       efile=pfile;
    }
 
+   /*
+      Handle the case when the user has not given us
+      get http://foobar.org (note the absense of the trailing /
+
+      It fixes segfault with a certain webserver which I've
+      seen ... (Geoffrey Lee <glee@gnupilgrims.org>).
+   */
+   if(*efile=='\0')
+      efile="/";
+
    Send("%s %s HTTP/1.1\r\n",method,efile);
    Send("Host: %s\r\n",ehost);
    if(user_agent && user_agent[0])
@@ -327,7 +340,7 @@ void Http::SendMethod(const char *method,const char *efile)
       if(!xstrcmp(referer,"."))
       {
 	 referer=GetConnectURL(NO_USER+NO_PASSWORD);
-	 if(referer[0] && referer[strlen(referer)-1]!='/')
+	 if(last_char(referer)!='/')
 	    slash="/";
       }
       if(referer && referer[0])
@@ -367,6 +380,7 @@ bool Http::ModeSupported()
    case QUOTE_CMD:
    case RENAME:
    case LIST:
+   case MP_LIST:
    case CHANGE_MODE:
       return false;
    case CONNECT_VERIFY:
@@ -465,6 +479,7 @@ void Http::SendRequest(const char *connection,const char *f)
       }
    case RENAME:
    case LIST:
+   case MP_LIST:
    case CHANGE_MODE:
       abort(); // unsupported
 
@@ -491,7 +506,7 @@ void Http::SendRequest(const char *connection,const char *f)
       else if(pos>0)
 	 Send("Range: bytes=%lld-%lld/%lld\r\n",(long long)pos,
 		     (long long)(entity_size-1),(long long)entity_size);
-      if(entity_date!=(time_t)-1)
+      if(entity_date!=NO_DATE)
       {
 	 char d[256];
 	 static const char weekday_names[][4]={
@@ -556,12 +571,14 @@ void Http::SendRequest(const char *connection,const char *f)
    chunk_pos=0;
    chunked_trailer=false;
    no_ranges=false;
+
+   send_buf->SetPos(0);
 }
 
 void Http::SendArrayInfoRequest()
 {
    int m=1;
-   if(keep_alive)
+   if(keep_alive && use_head)
    {
       m=keep_alive_max;
       if(m==-1)
@@ -692,6 +709,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       const char *filename=strstr(value,"filename=");
       if(!filename)
 	 return;
+      filename+=9;
       if(*filename=='"')
       {
 	 filename++;
@@ -714,6 +732,12 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       SetSuggestedFileName(filename);
       return;
    }
+   if(!strcasecmp(name,"Content-Type"))
+   {
+      xfree(entity_content_type);
+      entity_content_type=xstrdup(value);
+//       const char *cs=strstr(value,"charset=");
+   }
 }
 
 static
@@ -752,7 +776,7 @@ void Http::GetBetterConnection(int level)
       {
 	 if(level<2)
 	    continue;
-	 if(!connection_takeover || o->priority>=priority)
+	 if(!connection_takeover || (o->priority>=priority && !o->suspended))
 	    continue;
 	 o->Disconnect();
 	 return;
@@ -804,6 +828,11 @@ int Http::Do()
    case DISCONNECTED:
       if(mode==CLOSED || !hostname)
 	 return m;
+      if(mode==STORE && pos>0 && entity_size>=0 && pos>=entity_size)
+      {
+	 state=DONE;
+	 return MOVED;
+      }
       if(!hftp && mode==QUOTE_CMD && !post)
       {
       handle_quote_cmd:
@@ -896,7 +925,7 @@ int Http::Do()
       if(!NextTry())
       	 return MOVED;
 
-      sock=socket(peer[peer_curr].sa.sa_family,SOCK_STREAM,IPPROTO_TCP);
+      sock=SocketCreateTCP(peer[peer_curr].sa.sa_family);
       if(sock==-1)
       {
 	 if(peer_curr+1<peer_num)
@@ -1007,6 +1036,11 @@ int Http::Do()
 	 SetError(NOT_SUPP);
 	 return MOVED;
       }
+      if(mode==STORE && pos>0 && entity_size>=0 && pos>=entity_size)
+      {
+	 state=DONE;
+	 return MOVED;
+      }
       DebugPrint("---- ",_("Sending request..."));
       if(mode==ARRAY_INFO)
       {
@@ -1067,7 +1101,8 @@ int Http::Do()
 	 const char *eol=find_eol(buf,len,&eol_size);
 	 if(eol)
 	 {
-	    if(eol==buf)
+	    // empty line indicates end of headers.
+	    if(eol==buf && status)
 	    {
 	       DebugPrint("<--- ","",4);
 	       recv_buf->Skip(eol_size);
@@ -1106,16 +1141,16 @@ int Http::Do()
 		  status=0;
 		  status_code=0;
 		  if(array_for_info[array_ptr].get_time)
-		     array_for_info[array_ptr].time=(time_t)-1;
+		     array_for_info[array_ptr].time=NO_DATE;
 		  if(array_for_info[array_ptr].get_size)
-		     array_for_info[array_ptr].size=-1;
+		     array_for_info[array_ptr].size=NO_SIZE;
 		  if(++array_ptr>=array_cnt)
 		  {
 		     state=DONE;
 		     return MOVED;
 		  }
 		  // we can avoid reconnection if server supports it.
-		  if(keep_alive && (keep_alive_max>1 || keep_alive_max==-1))
+		  if(keep_alive && (keep_alive_max>1 || keep_alive_max==-1) && use_head)
 		  {
 		     SendArrayInfoRequest();
 		  }
@@ -1138,7 +1173,7 @@ int Http::Do()
 		  if(H_20X(status_code))
 		  {
 		     // should never happen
-		     DebugPrint("**** ",_("Success, but did nothing??"),0);
+		     DebugPrint("**** ","Success, but did nothing??",0);
 		     Disconnect();
 		     return MOVED;
 		  }
@@ -1167,15 +1202,18 @@ int Http::Do()
 		     &status_consumed,&status_code))
 	       {
 		  // simple 0.9 ?
-		  ver_major=0;
-		  ver_minor=9;
+		  proto_version=0x09;
 		  status_code=200;
-		  if(1!=sscanf(status,"HTTP %n%d",&status_consumed,&status_code))
+		  DebugPrint("**** ",_("Could not parse HTTP status line"),0);
+		  if(mode==STORE)
 		  {
-		     DebugPrint("**** ",_("Could not parse HTTP status line"),0);
-		     //FIXME: STORE
-		     goto pre_RECEIVING_BODY;
+		     state=DONE;
+		     Disconnect();
+		     state=DONE;
+		     return MOVED;
 		  }
+		  recv_buf->UnSkip(len+eol_size);
+		  goto pre_RECEIVING_BODY;
 	       }
 	       proto_version=(ver_major<<4)+ver_minor;
 
@@ -1298,7 +1336,7 @@ int Http::Do()
 	       sprintf(err,"%s (%s)",status+status_consumed,file);
 	    else
 	       sprintf(err,"%s (%s%s)",status+status_consumed,cwd,
-		  (cwd[0] && cwd[strlen(cwd)-1]=='/')?"":"/");
+				       (last_char(cwd)=='/')?"":"/");
 	 }
 	 state=RECEIVING_BODY;
 	 SetError(code,err);
@@ -1342,12 +1380,15 @@ int Http::Do()
       }
       else
       {
-	 recv_buf->Resume();
+	 if(recv_buf->IsSuspended())
+	 {
+	    recv_buf->Resume();
+	    if(recv_buf->Size()>0 || (recv_buf->Size()==0 && recv_buf->Eof()))
+	       m=MOVED;
+	 }
 	 BumpEventTime(send_buf->EventTime());
 	 BumpEventTime(recv_buf->EventTime());
-	 if(recv_buf->Size()>0 || (recv_buf->Size()==0 && recv_buf->Eof()))
-	    m=MOVED;
-	 else
+	 if(recv_buf->Size()==0)
 	 {
 	    // check if ranges were emulated by squid
 	    bool no_ranges_if_timeout=(bytes_received==0 && !seen_ranges_bytes);
@@ -1564,6 +1605,13 @@ int Http::Done()
    return IN_PROGRESS;
 }
 
+int Http::Buffered()
+{
+   if(mode!=STORE || !send_buf)
+      return 0;
+   return send_buf->Size()+SocketBuffered(sock);
+}
+
 int Http::Write(const void *buf,int size)
 {
    if(mode!=STORE)
@@ -1584,28 +1632,19 @@ int Http::Write(const void *buf,int size)
       if(size>allowed)
 	 size=allowed;
    }
-   if(size==0)
+   if(size+send_buf->Size()>=max_buf)
+      size=max_buf-send_buf->Size();
+   if(size<=0)
       return 0;
-   int res=write(sock,buf,size);
-   if(res==-1)
-   {
-      if(NonFatalError(errno))
-	 return DO_AGAIN;
-      if(NotSerious(errno) || errno==EPIPE)
-      {
-	 DebugPrint("**** ",strerror(errno),0);
-	 Disconnect();
-	 return error_code;
-      }
-      SetError(SEE_ERRNO,0);
-      Disconnect();
-      return error_code;
-   }
-   retries=0;
-   rate_limit->BytesPut(res);
-   pos+=res;
-   real_pos+=res;
-   return(res);
+
+   send_buf->Put((const char*)buf,size);
+
+   if(retries>0 && send_buf->GetPos()-send_buf->Size()>Buffered()+0x1000)
+      retries=0;
+   rate_limit->BytesPut(size);
+   pos+=size;
+   real_pos+=size;
+   return(size);
 }
 
 int Http::SendEOT()
@@ -1618,7 +1657,11 @@ int Http::SendEOT()
    {
       if(state==RECEIVING_HEADER && send_buf->Size()==0)
       {
-	 shutdown(sock,1);
+	 if(entity_size==NO_SIZE || pos<entity_size)
+	 {
+	    shutdown(sock,1);
+	    keep_alive=false;
+	 }
 	 sent_eot=true;
       	 return(OK);
       }
@@ -2139,7 +2182,7 @@ static void
 base64_encode (const char *s, char *store, int length)
 {
   /* Conversion table.  */
-  static char tbl[64] = {
+  static const char tbl[64] = {
     'A','B','C','D','E','F','G','H',
     'I','J','K','L','M','N','O','P',
     'Q','R','S','T','U','V','W','X',
