@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.h,v 1.35 2001/11/08 09:25:48 lav Exp $ */
+/* $Id: Http.h,v 1.40 2004/01/21 08:38:09 lav Exp $ */
 
 #ifndef HTTP_H
 #define HTTP_H
@@ -68,7 +68,7 @@ class Http : public NetAccess
    void SendArrayInfoRequest();
    int status_code;
    void HandleHeaderLine(const char *name,const char *value);
-   void GetBetterConnection(int level,int count);
+   void GetBetterConnection(int level);
    void SetCookie(const char *val);
    char *MakeCookie(const char *host,const char *path);
    void CookieMerge(char **c,const char *add);
@@ -116,6 +116,8 @@ class Http : public NetAccess
 
    const char *user_agent;
 
+   char *entity_content_type;
+
 protected:
    bool hftp;  // ftp over http proxy.
    bool https; // secure http
@@ -132,6 +134,7 @@ public:
 
    FileAccess *Clone() { return new Http(this); }
    static FileAccess *New();
+   FileSet *ParseLongList(const char *buf,int len,int *err=0);
 
    int Do();
    int Done();
@@ -139,6 +142,7 @@ public:
    int Write(const void *,int);
    int StoreStatus();
    int SendEOT();
+   int Buffered();
 
    void	ResetLocationData();
 
@@ -152,7 +156,7 @@ public:
 
    DirList *MakeDirList(ArgV *a);
    Glob *MakeGlob(const char *pattern);
-   ListInfo *MakeListInfo();
+   ListInfo *MakeListInfo(const char *path);
 
    void UseCache(bool use) { no_cache_this=!use; }
 
