@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.cc,v 1.53 2002/01/21 16:25:16 lav Exp $ */
+/* $Id: Resolver.cc,v 1.61 2004/05/28 07:57:05 lav Exp $ */
 
 #include <config.h>
 
@@ -49,21 +49,24 @@
 #include "log.h"
 #include "plural.h"
 
+#ifndef C_IN
+# define C_IN 1
+#endif
 #ifndef T_SRV
 # define T_SRV 33
 #endif
 
-#if !defined(HAVE_HSTRERROR_DECL)
+#if !HAVE_DECL_HSTRERROR
 extern "C" { const char *hstrerror(int); }
 #endif
 
 #ifdef HAVE_H_ERRNO
-#ifndef HAVE_H_ERRNO_DECL
+#if !HAVE_DECL_H_ERRNO
 CDECL int h_errno;
 #endif
 #endif
 
-#if defined(HAVE_RES_SEARCH) && !defined(HAVE_RES_SEARCH_DECL)
+#if HAVE_RES_SEARCH && !HAVE_DECL_RES_SEARCH
 CDECL int res_search(const char*,int,int,unsigned char*,int);
 #endif
 
@@ -88,7 +91,7 @@ static const address_family af_list[]=
    { -1, 0 }
 };
 
-ResolverCache *Resolver::cache=new ResolverCache;
+ResolverCache *Resolver::cache;
 
 Resolver::Resolver(const char *h,const char *p,const char *defp,
 		   const char *ser,const char *pr)
@@ -148,7 +151,7 @@ int   Resolver::Do()
 
    int m=STALL;
 
-   if(!no_cache)
+   if(!no_cache && cache)
    {
       const sockaddr_u *a;
       int n;
@@ -224,7 +227,7 @@ int   Resolver::Do()
       if(!buf)
       {
 	 Log::global->Format(4,"---- %s\n",_("Resolving host address..."));
-	 buf=new Buffer();
+	 buf=new IOBuffer(IOBuffer::GET);
 	 DoGethostbyname();
 	 if(deleting)
 	    return MOVED;
@@ -294,6 +297,8 @@ int   Resolver::Do()
    addr=(sockaddr_u*)xmalloc(n);
    memcpy(addr,s,n);
    done=true;
+   if(!cache)
+      cache=new ResolverCache;
    cache->Add(hostname,portname,defport,service,proto,addr,addr_num);
    Log::global->Format(4,plural("---- %d address$|es$ found\n",addr_num),addr_num);
    return MOVED;
@@ -319,14 +324,30 @@ void Resolver::AddAddress(int family,const char *address,int len)
    switch(family)
    {
    case AF_INET:
+      if(sizeof(add->in.sin_addr) != len)
+      {
+         addr_num--;
+         return;
+      }
       memcpy(&add->in.sin_addr,address,len);
       add->in.sin_port=port_number;
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
+      add->sa.sa_len=sizeof(add->in);
+#endif
       break;
 
 #if INET6
    case AF_INET6:
+      if(sizeof(add->in6.sin6_addr) != len)
+      {
+         addr_num--;
+         return;
+      }
       memcpy(&add->in6.sin6_addr,address,len);
       add->in6.sin6_port=port_number;
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
+      add->sa.sa_len=sizeof(add->in6);
+#endif
       break;
 #endif
 
@@ -734,6 +755,7 @@ void Resolver::LookupOne(const char *name)
 #  ifndef HAVE_H_ERRNO
 #   define HAVE_H_ERRNO 1
 #  endif
+#  undef h_errno // it could be a macro, but we want it to be local variable.
       int h_errno=0;
       ha=getipnodebyname(name,af,0,&h_errno);
 # elif defined(HAVE_GETHOSTBYNAME2)
@@ -754,6 +776,9 @@ void Resolver::LookupOne(const char *name)
 	 for(a=ha->h_addr_list; *a; a++)
 	    AddAddress(ha->h_addrtype, *a, ha->h_length);
 	 af_index++;
+# if defined(HAVE_GETIPNODEBYNAME)
+	 freehostent(ha);
+# endif
 	 continue;
       }
 
@@ -843,7 +868,8 @@ void Resolver::Reconfig(const char *name)
    timeout = ResMgr::Query("dns:fatal-timeout",hostname);
    if(!name || strncmp(name,"dns:",4))
       return;
-   cache->Clear();
+   if(cache)
+      cache->Clear();
 }
 
 
