@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,13 +29,13 @@ class SMTask
    virtual int Do() = 0;
 
    SMTask *next;
+   SMTask *prev_current;   // stack of running tasks
 
    static SMTask *chain;
-   static SMTask *sched_scan;
    static PollVec sched_total;
 
 protected:
-   bool	 running;
+   int	 running;
    bool	 suspended;
    bool	 deleting;
 
@@ -68,6 +68,8 @@ public:
    virtual void Reconfig(const char *name=0) {};
    static void ReconfigAll(const char *name);
 
+   virtual const char *GetLogContext() { return 0; }
+
    SMTask();
 
    static void Delete(SMTask *);
@@ -77,8 +79,24 @@ public:
 
    static SMTask *current;
 
+   static void Enter(SMTask *task)
+      {
+	 task->running++;
+	 task->prev_current=current;
+	 current=task;
+      }
+   static void Leave(SMTask *task)
+      {
+	 current=task->prev_current;
+	 task->prev_current=0;
+	 task->running--;
+      }
+   void Enter() { Enter(this); }
+   void Leave() { Leave(this); }
+
    static int TaskCount();
    static bool NonFatalError(int err);
+   static bool TemporaryNetworkError(int err);
 };
 
 class SMTaskInit : public SMTask
