@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SleepJob.cc,v 1.16 2002/01/04 12:31:30 lav Exp $ */
+/* $Id: SleepJob.cc,v 1.21 2004/02/17 10:40:56 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
@@ -107,7 +107,7 @@ int SleepJob::Do()
    return STALL;
 }
 
-void SleepJob::PrintStatus(int)
+void SleepJob::PrintStatus(int,const char *prefix)
 {
    if(repeat)
    {
@@ -127,11 +127,11 @@ void SleepJob::lftpMovesToBackground()
 }
 
 #define args (parent->args)
-#define eprintf (parent->eprintf)
+#define eprintf parent->eprintf
 #define session (parent->session)
 Job *cmd_sleep(CmdExec *parent)
 {
-   char *op=args->a0();
+   const char *op=args->a0();
    if(args->count()!=2)
    {
       eprintf(_("%s: argument required. "),op);
@@ -183,7 +183,7 @@ Job *cmd_at(CmdExec *parent)
    int date_len=0;
    for(;;)
    {
-      char *arg=args->getnext();
+      const char *arg=args->getnext();
       if(arg==0)
 	 break;
       if(!strcmp(arg,"--"))
@@ -208,7 +208,14 @@ Job *cmd_at(CmdExec *parent)
    xfree(date);
 
    if(when==0 || when==(time_t)-1)
+   {
+      const char *e=get_date_error();
+      eprintf("%s: %s\n",args->a0(),e?e:"unknown parse error");
       return 0;
+   }
+
+   if(when<now)
+      when+=86400;
 
    char *cmd=0;
    if(cmd_start)
