@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.187 2002/09/02 11:05:31 lav Exp $ */
+/* $Id: commands.cc,v 1.210 2004/05/28 11:48:12 lav Exp $ */
 
 #include <config.h>
 
@@ -59,7 +59,6 @@
 #include "GetPass.h"
 #include "SignalHook.h"
 #include "FileFeeder.h"
-#include "xalloca.h"
 #include "bookmark.h"
 #include "log.h"
 #include "module.h"
@@ -79,7 +78,6 @@
 #define HOUR   (60*MINUTE)
 #define DAY    (24*HOUR)
 
-Bookmark lftp_bookmarks;
 History	 cwd_history;
 
 CMD(alias); CMD(anon);   CMD(cd);      CMD(debug);	CMD(du);
@@ -94,7 +92,7 @@ CMD(ver);   CMD(close);  CMD(bookmark);CMD(lftp);
 CMD(echo);  CMD(suspend);CMD(sleep);   CMD(slot);
 CMD(at);    CMD(find);   CMD(command); CMD(module);
 CMD(lpwd);  CMD(glob);	 CMD(chmod);   CMD(queue);
-CMD(repeat);CMD(get1);   CMD(history);
+CMD(repeat);CMD(get1);   CMD(history); CMD(tasks);
 
 #ifdef MODULE_CMD_MIRROR
 # define cmd_mirror 0
@@ -185,7 +183,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	    " -i, --nocase         - case-insensitive pattern matching\n"
 	    " -I, --sortnocase     - sort names case-insensitively\n"
 	    " -D, --dirsfirst      - list directories first\n"
-	    "     --sort=OPT       - \"name\", \"size\"\n"
+	    "     --sort=OPT       - \"name\", \"size\", \"date\"\n"
 	    " -S                   - sort by file size\n"
 	    " --user, --group, --perms, --date, --linkcount, --links\n"
 	    "                      - show individual fields\n"
@@ -240,7 +238,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"fg",      cmd_wait,   0,"wait"},
    {"find",    cmd_find,0,
 	 N_("Usage: find [OPTS] [directory]\n"
-	 "Print contents of specified directory of current directory recursively.\n"
+	 "Print contents of specified directory or current directory recursively.\n"
 	 "Directories in the list are marked with trailing slash.\n"
 	 "You can redirect output of this command.\n"
 	 " -d, --maxdepth=LEVELS  Descend at most LEVELS of directories.\n")},
@@ -308,6 +306,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 N_("\nMirror specified remote directory to local directory\n\n"
 	 " -c, --continue         continue a mirror job if possible\n"
 	 " -e, --delete           delete files not present at remote site\n"
+	 "     --delete-first     delete old files before transferring new ones\n"
 	 " -s, --allow-suid       set suid/sgid bits according to remote site\n"
 	 "     --allow-chown      try to set owner and group on files\n"
 	 "     --ignore-time      ignore time when deciding whether to download\n"
@@ -317,13 +316,15 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 "     --no-umask         don't apply umask to file modes\n"
 	 " -R, --reverse          reverse mirror (put files)\n"
 	 " -L, --dereference      download symbolic links as files\n"
-	 " -N, --newer-than FILE  download only files newer than the file\n"
+	 " -N, --newer-than=SPEC  download only files newer than specified time\n"
 	 " -P, --parallel[=N]     download N files in parallel\n"
 	 " -i RX, --include RX    include matching files\n"
 	 " -x RX, --exclude RX    exclude matching files\n"
 	 "                        RX is extended regular expression\n"
 	 " -v, --verbose[=N]      verbose operation\n"
-	 "     --use-cache        use cached directory listings\n"
+	 "     --log=FILE         write lftp commands being executed to FILE\n"
+	 "     --script=FILE      write lftp commands to FILE, but don't execute them\n"
+	 "     --just-print, --dry-run    same as --script=-\n"
 	 "\n"
 	 "When using -R, the first directory is local and the second is remote.\n"
 	 "If the second directory is omitted, basename of first directory is used.\n"
@@ -469,6 +470,8 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"bzmore",   cmd_cat,    0,
 	 N_("Same as more, but filter each file through bzcat\n")},
 
+   {".tasks",  cmd_tasks,  0,0},
+
    {NULL,NULL}
 };
 
@@ -591,7 +594,8 @@ Job *CmdExec::builtin_cd()
    xfree(old_cwd);
    old_cwd=xstrdup(session->GetCwd());
 
-   if (!verify_path || background)
+   if(!verify_path || background
+   || (!verify_path_cached && LsCache::IsDirectory(session,dir)==1))
    {
       session->Chdir(dir,false);
       if(slot)
@@ -642,6 +646,7 @@ Job *CmdExec::builtin_lftp()
    int c;
    const char *cmd=0;
    char *acmd;
+   bool debug=false;
    static struct option lftp_options[]=
    {
       {"help",no_argument,0,'h'},
@@ -649,9 +654,8 @@ Job *CmdExec::builtin_lftp()
       {0,0,0,0}
    };
 
-   args->rewind();
    opterr=false;
-   while((c=args->getopt_long("+f:c:vh",lftp_options,0))!=EOF)
+   while((c=args->getopt_long("+f:c:vhd",lftp_options,0))!=EOF)
    {
       switch(c)
       {
@@ -673,12 +677,19 @@ Job *CmdExec::builtin_lftp()
 	 sprintf(acmd,"%s\n\n",optarg);
 	 cmd=acmd;
 	 break;
+      case('d'):
+	 debug=true;
+	 break;
       }
    }
    opterr=true;
 
    if(cmd)
+   {
       PrependCmd(cmd);
+      if(debug)
+	 PrependCmd("debug;");
+   }
 
    if(Done() && lftp_feeder)  // no feeder and no commands
    {
@@ -691,8 +702,10 @@ Job *CmdExec::builtin_lftp()
    if(!cmd)
    {
       /* if no lftp-specific options were found, call open */
+      args->rewind();
       return builtin_open();
    }
+   exit_code=0;
    return 0;
 }
 
@@ -702,19 +715,28 @@ Job *CmdExec::builtin_open()
 
    bool	 debug=false;
    char	 *port=NULL;
-   char	 *host=NULL;
+   const char *host=NULL;
    char  *path=NULL;
    char	 *user=NULL;
    char	 *pass=NULL;
    int	 c;
    NetRC::Entry *nrc=0;
    char  *cmd_to_exec=0;
-   char  *op=args->a0();
+   const char *op=args->a0();
    bool insecure=false;
    bool no_bm=false;
 
-   args->rewind();
-   while((c=args->getopt("u:p:e:dB"))!=EOF)
+   static struct option open_options[]=
+   {
+      {"port",required_argument,0,'p'},
+      {"user",required_argument,0,'u'},
+      {"execute",required_argument,0,'e'},
+      {"debug",optional_argument,0,'d'},
+      {"no-bookmark",no_argument,0,'B'},
+      {0,0,0,0}
+   };
+
+   while((c=args->getopt_long("u:p:e:dB",open_options,0))!=EOF)
    {
       switch(c)
       {
@@ -837,18 +859,18 @@ Job *CmdExec::builtin_open()
 	       path=uc.path;
 	 }
 
-	 if(!pass)
+	 // user gets substituted only if no proto is specified.
+	 if(!pass && (user || !uc.proto))
 	 {
-	    nrc=NetRC::LookupHost(host);
+	    nrc=NetRC::LookupHost(host,user);
 	    if(nrc)
 	    {
-	       if((!uc.proto && nrc->user && !user)
-	       || (nrc->user && user && !strcmp(nrc->user,user) && !pass))
-	       {
-		  user=nrc->user;
-		  if(nrc->pass)
-		     pass=nrc->pass;
-	       }
+	       if(!user)
+		  Log::global->Format(3,"---- using user `%s' and password from ~/.netrc\n",nrc->user);
+	       else
+		  Log::global->Format(3,"---- using password from ~/.netrc\n");
+	       user=nrc->user;
+	       pass=nrc->pass;
 	    }
 	 }
       }
@@ -867,7 +889,7 @@ Job *CmdExec::builtin_open()
 	       args->getarg(0));
 	 else
 	 {
-	    session->Login(user,0);
+	    session->Login(user,pass);
 	    // assume the new password is the correct one.
 	    session->SetPasswordGlobal(pass);
 	    session->InsecurePassword(insecure && !no_bm);
@@ -905,7 +927,10 @@ Job *CmdExec::builtin_open()
       PrependCmd("debug\n");
 
    if(url)
+   {
       delete url;
+      url=0;
+   }
 
    if(slot)
       ConnectionSlot::Set(slot,session);
@@ -930,24 +955,21 @@ Job *CmdExec::builtin_restart()
 Job *CmdExec::builtin_glob()
 {
    const char *op=args->a0();
-   bool files_only=true;
-   bool dirs_only=false;
    int opt;
+   GlobURL::type_select glob_type=GlobURL::FILES_ONLY;
 
    while((opt=args->getopt("+adf"))!=EOF)
    {
       switch(opt)
       {
       case('a'):
-	 files_only=dirs_only=false;
+	 glob_type=GlobURL::ALL;
 	 break;
       case('d'):
-	 dirs_only=true;
-	 files_only=false;
+	 glob_type=GlobURL::DIRS_ONLY;
 	 break;
       case('f'):
-	 files_only=true;
-	 dirs_only=false;
+	 glob_type=GlobURL::FILES_ONLY;
 	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),op);
@@ -973,11 +995,7 @@ Job *CmdExec::builtin_glob()
       args->rewind();
       return cmd_command(this);
    }
-   glob=new GlobURL(session,pat);
-   if(dirs_only)
-      glob->glob->DirectoriesOnly();
-   if(files_only)
-      glob->glob->FilesOnly();
+   glob=new GlobURL(session,pat,glob_type);
    builtin=BUILTIN_GLOB;
    return this;
 }
@@ -1000,11 +1018,7 @@ Job *CmdExec::builtin_queue()
    int pos = -1; /* default to the end */
    int verbose = -1; /* default */
 
-   args->rewind();
    int opt;
-
-   exit_code=1; // more failure exits than success exits, so set success explicitely
-
    while((opt=args->getopt_long("+dm:n:qvQw",queue_options,0))!=EOF)
    {
       switch(opt)
@@ -1132,7 +1146,7 @@ Job *CmdExec::builtin_queue()
 	  * queue -m 1 2  (move entry 1 to position 2)
 	  * queue -m "*get*" 1
 	  * queue -m 3    (move entry 3 to the end) */
-         char *a1 = args->getarg(args->getindex());
+         const char *a1 = args->getarg(args->getindex());
 	 if(a1 && !isdigit(a1[0])) {
 	    eprintf(_("%s: -m: Number expected as second argument. "), args->a0());
 	    goto err;
@@ -1323,7 +1337,6 @@ const char *FileSetOutput::parse_argv(ArgV *a)
       {0,0,0,0}
    };
 
-   a->rewind();
    int opt, longopt;
    while((opt=a->getopt_long(":1BdFhiklqsDIS", cls_options, &longopt))!=EOF)
    {
@@ -1332,6 +1345,8 @@ const char *FileSetOutput::parse_argv(ArgV *a)
 	 if(!strcmp(cls_options[longopt].name, "sort")) {
 	    if(!strcasecmp(optarg, "name")) sort = FileSet::BYNAME;
 	    else if(!strcasecmp(optarg, "size")) sort = FileSet::BYSIZE;
+	    else if(!strcasecmp(optarg, "date")) sort = FileSet::BYDATE;
+	    else if(!strcasecmp(optarg, "time")) sort = FileSet::BYDATE;
 	    else return _("invalid argument for `--sort'");
 	 } else if(!strcmp(cls_options[longopt].name, "filesize")) {
 	    size_filesonly = true;
@@ -1507,7 +1522,6 @@ CMD(get)
    bool reverse=false;
    char *output_dir=0;
 
-   args->rewind();
    if(!strncmp(op,"re",2))
    {
       cont=true;
@@ -1720,13 +1734,35 @@ CMD(mkdir)
 
 CMD(source)
 {
-   if(args->count()<2)
+   int opt;
+   bool e=false;
+   while((opt=args->getopt("+e"))!=EOF)
    {
-      // xgettext:c-format
-      eprintf(_("Usage: %s <file>\n"),args->a0());
-      return 0;
+      switch(opt)
+      {
+      case('e'):
+	 e=true;
+	 break;
+      case('?'):
+      usage:
+	 // xgettext:c-format
+	 eprintf(_("Usage: %s [-e] <file|command>\n"),args->a0());
+	 return 0;
+      }
    }
-   parent->SetCmdFeeder(new FileFeeder(new FileStream(args->getarg(1),O_RDONLY|O_ASCII)));
+   if(args->getindex()>=args->count())
+      goto usage;
+   FDStream *f=0;
+   if(e)
+   {
+      char *cmd=args->Combine(args->getindex());
+      f=new InputFilter(cmd);
+      xfree(cmd);
+   }
+   else
+      f=new FileStream(args->getarg(1),O_RDONLY|O_ASCII);
+
+   parent->SetCmdFeeder(new FileFeeder(f));
    exit_code=0;
    return 0;
 }
@@ -1734,7 +1770,6 @@ CMD(source)
 CMD(jobs)
 {
    int opt;
-   args->rewind();
    int v=1;
    while((opt=args->getopt("+v"))!=EOF)
    {
@@ -1762,7 +1797,6 @@ CMD(cd)
 CMD(pwd)
 {
    int opt;
-   args->rewind();
    int flags=0;
    while((opt=args->getopt("p"))!=EOF)
    {
@@ -1801,9 +1835,12 @@ CMD(debug)
    char	 *debug_file_name=0;
    int 	 fd=-1;
    bool  enabled=true;
+   bool	 show_pid=false;
+   bool	 show_time=false;
+   bool	 show_context=false;
 
    int opt;
-   while((opt=args->getopt("o:"))!=EOF)
+   while((opt=args->getopt("o:ptc"))!=EOF)
    {
       switch(opt)
       {
@@ -1820,6 +1857,15 @@ CMD(debug)
 	 fcntl(fd,F_SETFL,O_NONBLOCK);
 	 fcntl(fd,F_SETFD,FD_CLOEXEC);
 	 break;
+      case 'p':
+	 show_pid=true;
+	 break;
+      case 't':
+	 show_time=true;
+	 break;
+      case 'c':
+	 show_context=true;
+	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),op);
 	 return 0;
@@ -1831,7 +1877,7 @@ CMD(debug)
    else
       Log::global->SetOutput(fd,true);
 
-   char *a=args->getcurr();
+   const char *a=args->getcurr();
    if(a)
    {
       if(!strcasecmp(a,"off"))
@@ -1855,6 +1901,9 @@ CMD(debug)
    else
       Log::global->Disable();
 
+   Log::global->ShowPID(show_pid);
+   Log::global->ShowTime(show_time);
+   Log::global->ShowContext(show_context);
 
 #if 0
    if(interactive)
@@ -1877,8 +1926,8 @@ CMD(user)
       eprintf(_("Usage: %s userid [pass]\n"),args->getarg(0));
       return 0;
    }
-   char *user=args->getarg(1);
-   char	*pass=args->getarg(2);
+   const char *user=args->getarg(1);
+   const char *pass=args->getarg(2);
    bool insecure=(pass!=0);
 
    ParsedURL u(user,true);
@@ -1975,7 +2024,7 @@ CMD(kill)
    exit_code=0;
    for(;;)
    {
-      char *arg=args->getnext();
+      const char *arg=args->getnext();
       if(arg==0)
 	 break;
       if(!isdigit((unsigned char)arg[0]))
@@ -2003,7 +2052,6 @@ CMD(set)
    bool only_defaults=false;
    int c;
 
-   args->rewind();
    while((c=args->getopt("+ad"))!=EOF)
    {
       switch(c)
@@ -2020,7 +2068,8 @@ CMD(set)
       }
    }
    args->back();
-   char *a=args->getnext();
+   const char *ac=args->getnext();
+   char *a=alloca_strdup(ac);
 
    if(a==0)
    {
@@ -2092,15 +2141,14 @@ CMD(alias)
 
 CMD(wait)
 {
-   char *op=args->a0();
+   const char *op=args->a0();
    if(args->count()>2)
    {
       eprintf(_("Usage: %s [<jobno>]\n"),op);
       return 0;
    }
    int n=-1;
-   args->rewind();
-   char *jn=args->getnext();
+   const char *jn=args->getnext();
    if(jn)
    {
       if(!strcasecmp(jn,"all"))
@@ -2153,7 +2201,7 @@ CMD(subsh)
 {
    CmdExec *e=new CmdExec(session->Clone(),parent->cwd->Clone());
 
-   char *c=args->getarg(1);
+   const char *c=args->getarg(1);
    e->FeedCmd(c);
    e->FeedCmd("\n");
    e->cmdline=(char*)xmalloc(strlen(c)+3);
@@ -2180,7 +2228,6 @@ const char *const cache_subcmd[]={
 
 CMD(cache)  // cache control
 {
-   args->rewind();
    const char *op=args->getnext();
 
    if(!op)
@@ -2255,7 +2302,7 @@ CMD(scache)
    }
    else
    {
-      char *a=args->getarg(1);
+      const char *a=args->getarg(1);
       if(!isdigit((unsigned char)a[0]))
       {
 	 eprintf(_("%s: %s - not a number\n"),args->a0(),a);
@@ -2337,10 +2384,9 @@ CMD(help)
 {
    if(args->count()>1)
    {
-      args->rewind();
       for(;;)
       {
-	 char *cmd=args->getnext();
+	 const char *cmd=args->getnext();
 	 if(cmd==0)
 	    break;
 	 parent->print_cmd_help(cmd);
@@ -2350,13 +2396,14 @@ CMD(help)
 
    parent->print_cmd_index();
 
+   exit_code=0;
    return 0;
 }
 
 CMD(ver)
 {
    printf(
-      _("Lftp | Version %s | Copyright (c) 1996-2002 Alexander V. Lukyanov\n"),VERSION);
+      _("Lftp | Version %s | Copyright (c) 1996-2004 Alexander V. Lukyanov\n"),VERSION);
    printf(
       _("This is free software with ABSOLUTELY NO WARRANTY. See COPYING for details.\n"));
    printf(
@@ -2397,7 +2444,6 @@ static ResDecl res_save_passwords
 
 CMD(bookmark)
 {
-   args->rewind();
    const char *op=args->getnext();
 
    if(!op)
@@ -2443,7 +2489,7 @@ CMD(bookmark)
 	    char *a=string_alloca(strlen(value)*3+2);
 	    // encode some more characters, special to CmdExec parser.
 	    url::encode_string(value,a,"&;|\"'\\");
-	    if(value[0] && value[strlen(value)-1]!='/')
+	    if(value[0] && last_char(value)!='/')
 	       strcat(a,"/");
 	    value=a;
 	 }
@@ -2473,6 +2519,7 @@ CMD(bookmark)
    }
    else if(!strcasecmp(op,"edit"))
    {
+      lftp_bookmarks.Remove(""); // force bookmark file creation
       parent->PrependCmd("shell \"/bin/sh -c 'exec ${EDITOR:-vi} $HOME/.lftp/bookmarks'\"\n");
    }
    else if(!strcasecmp(op,"import"))
@@ -2536,7 +2583,6 @@ CMD(find)
    int maxdepth = -1;
    const char *op=args->a0();
 
-   args->rewind();
    while((opt=args->getopt_long("+d:",find_options,&longopt))!=EOF)
    {
       switch(opt)
@@ -2601,7 +2647,6 @@ CMD(du)
 
    const char *op=args->a0();
 
-   args->rewind();
    int opt, longopt;
    while((opt=args->getopt_long("+abcd:FhHkmsS",du_options,&longopt))!=EOF)
    {
@@ -2736,7 +2781,7 @@ CMD(command)
 
 CMD(module)
 {
-   char *op=args->a0();
+   const char *op=args->a0();
    if(args->count()<2)
    {
       eprintf(_("Usage: %s module [args...]\n"),args->a0());
@@ -2826,9 +2871,10 @@ CMD(chmod)
    if(modeind == 0)
       modeind = args->getindex();
 
-   char *arg = alloca_strdup(args->getarg(modeind));
+   const char *arg = args->getarg(modeind);
    if(!arg)
       goto usage;
+   arg = alloca_strdup(arg);
    args->delarg(modeind);
 
    if(!args->getcurr())
@@ -2862,8 +2908,8 @@ CMD(get1)
    static struct option get1_options[]=
    {
       {"ascii",no_argument,0,'a'},
-      {"source-region",required_argument,0,'r'},
-      {"target-region",required_argument,0,'R'},
+      {"source-region",required_argument,0,256+'r'},
+      {"target-position",required_argument,0,256+'R'},
       {"continue",no_argument,0,'c'},
       {"output",required_argument,0,'o'},
       {"remove-source-later",no_argument,0,'E'},
@@ -2875,8 +2921,10 @@ CMD(get1)
    const char *dst=0;
    bool cont=false;
    bool ascii=false;
+   long long source_region_begin=0,source_region_end=FILE_END;
+   long long target_region_begin=0,target_region_end=FILE_END;
+   int n,p;
 
-   args->rewind();
    while((opt=args->getopt_long("arco:",get1_options,0))!=EOF)
    {
       switch(opt)
@@ -2890,8 +2938,23 @@ CMD(get1)
       case 'o':
 	 dst=optarg;
 	 break;
-      case 'r':
-	 // FIXME
+      case 256+'r':
+	 source_region_end=FILE_END;
+	 n=sscanf(optarg,"%lld%n-%lld",&source_region_begin,&p,&source_region_end);
+	 if(n<1 || (n==1 && (optarg[p] && (optarg[p]!='-' || optarg[p+1]))))
+	 {
+	    eprintf("%s\n",_("Invalid range format. Format is min-max, e.g. 10-20."));
+	    goto usage;
+	 }
+	 break;
+      case 256+'R':
+	 target_region_end=FILE_END;
+	 n=sscanf(optarg,"%lld",&target_region_begin);
+	 if(n<1)
+	 {
+	    eprintf("%s\n",_("Invalid range format. Format is min-max, e.g. 10-20."));
+	    goto usage;
+	 }
 	 break;
       case '?':
       usage:
@@ -2911,17 +2974,16 @@ CMD(get1)
    }
    else
    {
-      if(dst[strlen(dst)-1]=='/')
+      if(last_char(dst)=='/' && basename_ptr(dst)[0]!='/')
       {
-	 const char *slash=strrchr(src,'/');
-	 if(slash)
-	    slash++;
-	 else
-	    slash=src;
-	 char *dst1=string_alloca(strlen(dst)+strlen(slash)+1);
-	 strcpy(dst1,dst);
-	 strcat(dst1,slash);
-	 dst=dst1;
+	 const char *bn=basename_ptr(src);
+	 if(bn[0]!='/')
+	 {
+	    char *dst1=string_alloca(strlen(dst)+strlen(bn)+1);
+	    strcpy(dst1,dst);
+	    strcat(dst1,bn);
+	    dst=dst1;
+	 }
       }
    }
 
@@ -2954,11 +3016,15 @@ CMD(get1)
    FileCopyPeer *dst_peer=0;
 
    src_peer=FileCopyPeerFA::New(session->Clone(),src,FA::RETRIEVE,true);
+   if(!cont && (source_region_begin>0 || source_region_end!=FILE_END))
+      src_peer->SetRange(source_region_begin,source_region_end);
 
    if(dst_url.proto==0)
-      dst_peer=FileCopyPeerFDStream::NewPut(dst,cont);
+      dst_peer=FileCopyPeerFDStream::NewPut(dst,cont||target_region_begin>0);
    else
       dst_peer=new FileCopyPeerFA(&dst_url,FA::STORE);
+   if(!cont && (target_region_begin>0 || target_region_end!=FILE_END))
+      dst_peer->SetRange(target_region_begin,target_region_end);
 
    FileCopy *c=FileCopy::New(src_peer,dst_peer,cont);
 
@@ -2982,7 +3048,6 @@ CMD(history)
    };
 
    exit_code=0;
-   args->rewind();
    int opt;
    while((opt=args->getopt_long("+r:w:cl",history_options,0))!=EOF) {
       switch(opt) {
@@ -3007,7 +3072,7 @@ CMD(history)
    }
 
    int cnt = 16;
-   if(char *arg = args->getcurr()) {
+   if(const char *arg = args->getcurr()) {
       if(!strcasecmp(arg, "all"))
 	 cnt = -1;
       else if(isdigit(arg[0]))
@@ -3062,3 +3127,10 @@ CMD(slot)
       return j;
    }
 }
+
+CMD(tasks)
+{
+   printf("task_count=%d\n",SMTask::TaskCount());
+   exit_code=0;
+   return 0;
+}
