@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: complete.cc,v 1.55.2.1 2002/01/15 16:20:24 lav Exp $ */
+/* $Id: complete.cc,v 1.66 2003/12/17 13:00:23 lav Exp $ */
 
 #include <config.h>
 
@@ -30,7 +30,6 @@
 #include <glob.h>
 #include <sys/time.h>
 #include <assert.h>
-#include "xalloca.h"
 #include "xmalloc.h"
 #include "FileAccess.h"
 #include "CmdExec.h"
@@ -44,6 +43,7 @@
 #include "ResMgr.h"
 #include "ColumnOutput.h"
 #include "FileSetOutput.h"
+#include "OutputJob.h"
 #include "misc.h"
 
 CDECL_BEGIN
@@ -85,7 +85,9 @@ char *command_generator(const char *text,int state)
    while ((name=CmdExec::CmdByIndex(cindex)->name)!=0)
    {
       cindex++;
-      if(strncmp(name,text,len)==0)
+      if(name[0]=='.' && len==0)
+	 continue;   // skip hidden commands
+      if(strncasecmp(name,text,len)==0)
 	 return(xstrdup(name));
    }
 
@@ -93,7 +95,7 @@ char *command_generator(const char *text,int state)
    {
       const Alias *tmp=alias;
       alias=alias->next;
-      if(strncmp(tmp->alias,text,len)==0)
+      if(strncasecmp(tmp->alias,text,len)==0)
          return(xstrdup(tmp->alias));
    }
 
@@ -101,11 +103,8 @@ char *command_generator(const char *text,int state)
    return(NULL);
 }
 
-/* this is used for both remote and local gen now; rename */
 static char *file_generator(const char *text,int state)
 {
-   const char *name;
-
    /* If this is a new word to complete, initialize now. */
    if(!state)
       cindex=0;
@@ -115,7 +114,8 @@ static char *file_generator(const char *text,int state)
 
    while((*glob_res)[cindex])
    {
-      name=(*glob_res)[cindex++]->name;
+      const char *name=(*glob_res)[cindex++]->name;
+
       if(!name[0])
 	 continue;
       /* it's not our job to be case-sensitive; that's done (or not done)
@@ -393,6 +393,11 @@ static completion_type cmd_completion_type(const char *cmd,int start)
 	 was_N=true;
 	 break;
       }
+      if(i-14 >= 0 && !strncmp(rl_line_buffer+i-13, "--newer-than",12) && isspace(rl_line_buffer[i-14]))
+      {
+	 was_N=true;
+	 break;
+      }
       if(!strncmp(rl_line_buffer+i-3,"-O",2) && isspace(rl_line_buffer[i-4]))
       {
 	 was_O=true;
@@ -672,7 +677,8 @@ static char **lftp_completion (const char *text,int start,int end)
 
       SignalHook::ResetCount(SIGINT);
       glob_res=NULL;
-      rg=new GlobURL(completion_shell->session,pat);
+      rg=new GlobURL(completion_shell->session,pat,
+		     type==REMOTE_DIR?GlobURL::DIRS_ONLY:GlobURL::ALL);
 
       rl_save_prompt();
 
@@ -688,8 +694,6 @@ static char **lftp_completion (const char *text,int start,int end)
 	    rg->glob->CaseFold();
 	 } else
 	    rl_variable_bind("completion-ignore-case", "0");
-	 if(type==REMOTE_DIR)
-	    rg->glob->DirectoriesOnly();
 
 	 Timer status_timer;
 	 status_timer.SetMilliSeconds(20);
@@ -874,7 +878,7 @@ backslash_quote (char *string)
       switch (c)
 	{
  	case '(': case ')':
- 	case '!': case '{': case '}':		/* reserved words */
+ 	case '{': case '}':			/* reserved words */
  	case '^':
  	case '$': case '`':			/* expansion chars */
 	  if(!shell_cmd)
@@ -885,7 +889,7 @@ backslash_quote (char *string)
 	case ' ': case '\t': case '\n':		/* IFS white space */
 	case '"': case '\'': case '\\':		/* quoting chars */
 	case '|': case '&': case ';':		/* shell metacharacters */
-	case '<': case '>':
+	case '<': case '>': case '!':
 	  *r++ = '\\';
 	  *r++ = c;
 	  break;
@@ -1143,7 +1147,6 @@ int lftp_char_is_quoted(char *string,int eindex)
 extern "C" int (*rl_last_func)(int,int);
 static int lftp_complete_remote(int count,int key)
 {
-
    if(rl_last_func == lftp_complete_remote)
       rl_last_func = rl_complete;
 
@@ -1177,6 +1180,8 @@ int lftp_rl_getc(FILE *file)
    }
 }
 
+extern int lftp_slot(int,int);
+
 /* Tell the GNU Readline library how to complete.  We want to try to complete
    on command names if this is the first word in the line, or on filenames
    if not. */
@@ -1188,19 +1193,27 @@ void lftp_readline_init ()
       lftp_rl_getc,	      // rl_getc_function
       "\"'",		      // rl_completer_quote_characters
       " \t\n\"'",	      // rl_completer_word_break_characters
-      " \t\n\\\"'>;|&()*?[]~",// rl_filename_quote_characters
+      " \t\n\\\"'>;|&()*?[]~!",// rl_filename_quote_characters
       bash_quote_filename,    // rl_filename_quoting_function
       bash_dequote_filename,  // rl_filename_dequoting_function
       lftp_char_is_quoted);   // rl_char_is_quoted_p
 
    lftp_rl_add_defun("complete-remote",lftp_complete_remote,-1);
    lftp_rl_bind("Meta-Tab","complete-remote");
+
+   lftp_rl_add_defun("slot-change",lftp_slot,-1);
+   char key[7];
+   strcpy(key,"Meta-N");
+   for(int i=0; i<10; i++)
+   {
+      key[5]='0'+i;
+      lftp_rl_bind(key,"slot-change");
+   }
 }
 
 extern "C" void completion_display_list (char **matches, int len)
 {
-   FDStream o(1,"<stdout>");
-   Buffer *b = new Buffer;
+   OutputJob *b=new OutputJob((FDStream *) NULL, "completion");
 
    if(glob_res) {
       /* Our last completion action was of files, and we kept that
@@ -1216,7 +1229,7 @@ extern "C" void completion_display_list (char **matches, int len)
       }
 
       FileSetOutput fso;
-      fso.config(&o);
+      fso.config(b);
 
       ArgV arg("", ResMgr::Query("cmd:cls-completion-default", 0));
       fso.parse_argv(&arg);
@@ -1229,19 +1242,19 @@ extern "C" void completion_display_list (char **matches, int len)
 	 c.append();
 	 c.add(matches[i], "");
       }
-      c.print(b, fd_width(o.getfd()), isatty(o.getfd()));
+      c.print(b, b->GetWidth(), b->IsTTY());
    }
 
-   const char *buf;
-   int sz;
-   b->Get(&buf, &sz);
+   b->PutEOF();
 
-   while(sz) {
-      int ret = write(1, buf, sz);
-      if(ret <= 0)
-	 break; /* oops */
-      sz -= ret;
-      buf += ret;
+   while(!b->Done())
+   {
+      SMTask::Schedule();
+      if(SignalHook::GetCount(SIGINT))
+      {
+	 SignalHook::ResetCount(SIGINT);
+	 break;
+      }
    }
 
    SMTask::Delete(b);
