@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.56 2002/08/05 08:51:14 lav Exp $ */
+/* $Id: misc.cc,v 1.66 2004/05/28 07:57:06 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -41,14 +41,17 @@
 # include <sys/time.h>
 #endif
 
+CDECL_BEGIN
 #include <regex.h>
+CDECL_END
 #include "misc.h"
 #include "ProcWait.h"
 #include "SignalHook.h"
 #include "url.h"
 #include "ResMgr.h"
+#include <mbswidth.h>
 
-#if defined(HAVE_UNSETENV) && !defined(HAVE_UNSETENV_DECL)
+#if HAVE_UNSETENV && !HAVE_DECL_UNSETENV
 CDECL void unsetenv(const char *name);
 #endif
 
@@ -56,6 +59,8 @@ const char *dir_file(const char *dir,const char *file)
 {
    if(dir==0 || dir[0]==0)
       return file?file:dir;
+   if(file && file[0]=='.' && file[1]=='/')
+      file+=2;
    if(file==0 || file[0]==0)
       return dir;
    if(file[0]=='/')
@@ -433,7 +438,7 @@ int guess_year(int month,int day,int hour,int minute)
    const struct tm &now=SMTask::now;
    int year=now.tm_year+1900;
    if(((month     *32+        day)*64+       hour)*64+       minute
-    > ((now.tm_mon*32+now.tm_mday)*64+now.tm_hour)*64+now.tm_min)
+    > ((now.tm_mon*32+now.tm_mday)*64+now.tm_hour)*64+now.tm_min+5)
       year--;
    return year;
 }
@@ -449,28 +454,39 @@ const char *squeeze_file_name(const char *name,int w)
 {
    static char *buf;
    static int buf_len;
+   int mbflags=MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE;
 
-   int name_len=strlen(name);
+   int name_width=mbswidth(name,mbflags);
 
-   if(name_len<=w)
+   if(name_width<=w)
       return name;
 
-   if(buf_len<w+20)
-      buf=(char*)xrealloc(buf,buf_len=w+20);
+   if(buf_len<w*4+20)
+      buf=(char*)xrealloc(buf,buf_len=w*4+20);
 
    const char *b=basename_ptr(name);
-   int b_len=name_len-(b-name);
-   if(b_len<=w-4 && b_len>w-15)
+   int b_width=name_width-mbsnwidth(name,b-name,mbflags);
+   if(b_width<=w-4 && b_width>w-15)
    {
       strcpy(buf,".../");
       strcat(buf,b);
       return buf;
    }
-   b=name+name_len-w;
-   if(w<4)
-      return b;
-   strcpy(buf,"...");
-   strcat(buf,b+3);
+   int b_len=strlen(b);
+   while(b_width>(w<3?w-1:w-3) && b_len>0)
+   {
+      int ch_len=mblen(b,b_len);
+      if(ch_len<1)
+	 ch_len=1;
+      b_width-=mbsnwidth(b,ch_len,mbflags);
+      b+=ch_len;
+      b_len-=ch_len;
+   }
+   if(w>=6)
+      strcpy(buf,"...");
+   else
+      strcpy(buf,"<");
+   strcat(buf,b);
    return buf;
 }
 
@@ -524,13 +540,21 @@ void unsetenv(const char *env)
 
 static void set_tz(const char *tz)
 {
+   static char *put_tz;
+   static int put_tz_alloc;
+
    if(!tz)
    {
       unsetenv("TZ");
+
+      xfree(put_tz);
+      put_tz=0;
+      put_tz_alloc=0;
+
+      tzset();
       return;
    }
-   static char *put_tz;
-   static int put_tz_alloc;
+
    int tz_len=strlen(tz)+4;
    char *new_tz=put_tz;
    if(tz_len>put_tz_alloc)
@@ -542,6 +566,8 @@ static void set_tz(const char *tz)
       xfree(put_tz);
       put_tz=new_tz;
    }
+   // now initialize libc variables from env TZ.
+   tzset();
 }
 static char *saved_tz=0;
 static void save_tz()
@@ -557,6 +583,8 @@ time_t mktime_from_tz(struct tm *t,const char *tz)
 {
    if(!tz || !*tz)
       return mktime(t);
+   if(!strcasecmp(tz,"GMT"))
+      return mktime_from_utc(t);
    if(isdigit((unsigned char)*tz) || *tz=='+' || *tz=='-')
    {
       char *tz1=string_alloca(strlen(tz)+4);
@@ -765,12 +793,16 @@ char *xasprintf(const char *format, ...)
  * file/name -> "file"
  * file/name/ -> "file"
  * file -> ""
- * note: last differs from dirname (wwould return ".")
+ * note: the last differs from dirname(1) (which would return ".")
  *
  */
 char *dirname_alloc(const char *fn)
 {
    char *ret=xstrdup(fn);
+   return dirname_modify(ret);
+}
+char *dirname_modify(char *ret)
+{
    int len=strlen(ret);
 
    /* remove trailing slashes */
@@ -786,3 +818,9 @@ char *dirname_alloc(const char *fn)
       *slash=0;
    return ret;
 }
+
+char last_char(const char *str)
+{
+   int len=strlen(str);
+   return str[len-(len>0)];
+}
