@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.h,v 1.27.2.1 2002/01/15 16:20:26 lav Exp $ */
+/* $Id: misc.h,v 1.34 2004/04/16 14:27:36 lav Exp $ */
 
 #ifndef MISC_H
 #define MISC_H
@@ -80,7 +80,7 @@ int parse_month(const char *);
 int parse_perms(const char *);
 const char *format_perms(int p);
 int parse_year_or_time(const char *year_or_time,int *year,int *hour,int *minute);
-int guess_year(int month,int day,int hour,int minute,const struct tm *now);
+int guess_year(int month,int day,int hour,int minute);
 
 time_t mktime_from_utc(const struct tm *);
 time_t mktime_from_tz(struct tm *,const char *tz);
@@ -109,7 +109,10 @@ char *xstrftime(const char *format, const struct tm *tm);
 char *xvasprintf(const char *format, va_list ap);
 char *xasprintf(const char *format, ...);
 
-/* convert \r\n -> \n in buf; return new size */
-int crlf_to_lf(char *buf, int s);
+char *dirname_alloc(const char *fn);
+char *dirname_modify(char *fn);
+
+/* returns last character of string or \0 if string is empty */
+char last_char(const char *str);
 
 #endif // MISC_H
