@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,15 +22,20 @@
 #include "trio.h"
 #include "xstring.h"
 #include <ctype.h>
-#include "xalloca.h"
 #include "url.h"
 #include "ascii_ctype.h"
+#include "ConnectionSlot.h"
+#include "bookmark.h"
+#include "misc.h"
 
 /*
    URL -> [PROTO://]CONNECT[[:]/PATH]
    CONNECT -> [USER[:PASS]@]HOST[:PORT]
 */
 
+static bool valid_slot(const char *s);
+static bool valid_bm(const char *s);
+
 ParsedURL::ParsedURL(const char *url,bool proto_required,bool use_rfc1738)
 {
    memory=(char*)xmalloc(strlen(url)*2+20+1);
@@ -73,6 +78,23 @@ ParsedURL::ParsedURL(const char *url,bool proto_required,bool use_rfc1738)
       path=scan+10;
       goto decode;
    }
+   else if(scan[0]==':'
+   && ((!strncmp(base,"slot:",5) && valid_slot(scan+1))
+       || (!strncmp(base,"bm:",3) && valid_bm(scan+1))))
+   {
+      // special form for selecting a connection slot or a bookmark
+      *scan++=0;
+      proto=base;
+      host=scan;
+      scan=strchr(scan,'/');
+      if(scan)
+      {
+	 memmove(scan+1,scan,strlen(scan)+1);
+	 *scan++=0;
+	 path=scan;
+      }
+      goto decode;
+   }
    else if(proto_required)
    {
       // all the rest is path, if protocol is required.
@@ -183,6 +205,94 @@ decode:
    url::decode_string(pass);
    url::decode_string(host);
    url::decode_string(path);
+
+   FileAccess *fa=0;
+   if(!xstrcmp(proto,"slot"))
+   {
+      fa=ConnectionSlot::FindSession(host);
+      if(!fa)
+	 return;
+      xfree(orig_url);
+      orig_url=0;
+      char *orig_path=alloca_strdup(path);
+      xfree(memory);
+      memory=(char*)xmalloc(
+		     xstrlen(fa->GetProto())+1
+		    +xstrlen(fa->GetUser())+1
+		    +xstrlen(fa->GetPassword())+1
+		    +xstrlen(fa->GetHostName())+1
+		    +xstrlen(fa->GetPort())+1
+		    +xstrlen(fa->GetCwd())+2
+		    +xstrlen(orig_path)+1);
+      proto=user=pass=host=port=path=0;
+      char *next=memory;
+      proto=next;
+      strcpy(proto,fa->GetProto());
+      next=proto+strlen(proto)+1;
+      if(fa->GetUser())
+      {
+	 user=next;
+	 strcpy(user,fa->GetUser());
+	 next=user+strlen(user)+1;
+      }
+      if(fa->GetPassword())
+      {
+	 pass=next;
+	 strcpy(pass,fa->GetPassword());
+	 next=pass+strlen(pass)+1;
+      }
+      if(fa->GetHostName())
+      {
+	 host=next;
+	 strcpy(host,fa->GetHostName());
+	 next=host+strlen(host)+1;
+      }
+      if(fa->GetPort())
+      {
+	 port=next;
+	 strcpy(port,fa->GetPort());
+	 next=port+strlen(port)+1;
+      }
+      if(fa->GetCwd())
+      {
+	 path=next;
+	 orig_path+=(orig_path!=0 && orig_path[0]=='/');
+	 strcpy(path,dir_file(fa->GetCwd(),orig_path));
+	 if(!orig_path || orig_path[0]==0)
+	    strcat(path,"/");
+      }
+   }
+   else if(!xstrcmp(proto,"bm"))
+   {
+      const char *bm=lftp_bookmarks.Lookup(host);
+      if(!bm || !valid_bm(host))
+	 return;
+      ParsedURL bu(url_file(bm,path+(path && path[0]=='/')));
+      // move the data
+      xfree(memory);
+      xfree(orig_url);
+      memcpy(this,&bu,sizeof(bu));
+      bu.memory=0;
+      bu.orig_url=0; // so that dtor won't free them
+   }
+}
+
+static bool valid_slot(const char *cs)
+{
+   char *s=alloca_strdup(cs);
+   char *slash=strchr(s,'/');
+   if(slash)
+      *slash=0;
+   return 0!=ConnectionSlot::Find(s);
+}
+static bool valid_bm(const char *bm)
+{
+   char *s=alloca_strdup(bm);
+   char *slash=strchr(s,'/');
+   if(slash)
+      *slash=0;
+   const char *url=lftp_bookmarks.Lookup(s);
+   return(url && !strchr(url,' ') && !strchr(url,'\t'));
 }
 
 int url::path_index(const char *base)
@@ -206,8 +316,26 @@ int url::path_index(const char *base)
    return 0;
 }
 
-void ParsedURL::Combine(char *url,const char *home,bool use_rfc1738)
+char *ParsedURL::Combine(const char *home,bool use_rfc1738)
 {
+   int len=1;
+   if(proto)
+      len+=strlen(proto)+strlen("://");
+   if(user)
+   {
+      len+=strlen(user)*3+1;
+      if(pass)
+	 len+=strlen(pass)*3+1;
+   }
+   if(host)
+      len+=strlen(host)*3;
+   if(port)
+      len+=1+strlen(port)*3;
+   if(path)
+      len+=1+strlen(path)*3;
+
+   char *url=(char*)xmalloc(len);
+
    bool is_file=!xstrcmp(proto,"file");
    bool is_ftp=(!xstrcmp(proto,"ftp") || !xstrcmp(proto,"hftp"));
 
@@ -235,7 +363,7 @@ void ParsedURL::Combine(char *url,const char *home,bool use_rfc1738)
       url::encode_string(port,url+strlen(url),URL_PORT_UNSAFE);
    }
    if(path==0)
-      return;
+      return url;
    if(strcmp(path,"~"))
    {
       if(path[0]!='/' && !is_file) // e.g. ~/path
@@ -254,6 +382,7 @@ void ParsedURL::Combine(char *url,const char *home,bool use_rfc1738)
       }
       url::encode_string(path+p_offset,url+strlen(url),URL_PATH_UNSAFE);
    }
+   return url;
 }
 
 void url::decode_string(char *p)
